@@ -364,6 +364,7 @@ static int insert_delete_side_of_split(
 	memset(&deleted->new_file, 0, sizeof(deleted->new_file));
 	deleted->new_file.path = deleted->old_file.path;
 	deleted->new_file.flags |= GIT_DIFF_FLAG_VALID_ID;
+	git_oid_clear(&deleted->new_file.id, diff->opts.oid_type);
 
 	return git_vector_insert(onto, deleted);
 }
@@ -397,6 +398,7 @@ static int apply_splits_and_deletes(
 			memset(&delta->old_file, 0, sizeof(delta->old_file));
 			delta->old_file.path = delta->new_file.path;
 			delta->old_file.flags |= GIT_DIFF_FLAG_VALID_ID;
+			git_oid_clear(&delta->old_file.id, diff->opts.oid_type);
 		}
 
 		/* clean up delta before inserting into new list */
@@ -651,6 +653,23 @@ static int calc_self_similarity(
 	return 0;
 }
 
+static void handle_non_blob(
+	git_diff *diff,
+	const git_diff_find_options *opts,
+	size_t delta_idx)
+{
+	git_diff_delta *delta = GIT_VECTOR_GET(&diff->deltas, delta_idx);
+
+	/* skip things that are blobs */
+	if (GIT_MODE_ISBLOB(delta->old_file.mode))
+		return;
+
+	/* honor "remove unmodified" flag for non-blobs (eg submodules) */
+	if (delta->status == GIT_DELTA_UNMODIFIED &&
+	    FLAG_SET(opts, GIT_DIFF_FIND_REMOVE_UNMODIFIED))
+		delta->flags |= GIT_DIFF_FLAG__TO_DELETE;
+}
+
 static bool is_rename_target(
 	git_diff *diff,
 	const git_diff_find_options *opts,
@@ -808,7 +827,8 @@ int git_diff_find_similar(
 	git_diff_find_options opts = GIT_DIFF_FIND_OPTIONS_INIT;
 	size_t num_deltas, num_srcs = 0, num_tgts = 0;
 	size_t tried_srcs = 0, tried_tgts = 0;
-	size_t num_rewrites = 0, num_updates = 0, num_bumped = 0;
+	size_t num_rewrites = 0, num_updates = 0, num_bumped = 0,
+	       num_to_delete = 0;
 	size_t sigcache_size;
 	void **sigcache = NULL; /* cache of similarity metric file signatures */
 	diff_find_match *tgt2src = NULL;
@@ -842,6 +862,8 @@ int git_diff_find_similar(
 	 * mark them for splitting if break-rewrites is enabled
 	 */
 	git_vector_foreach(&diff->deltas, t, tgt) {
+		handle_non_blob(diff, &opts, t);
+
 		if (is_rename_source(diff, &opts, t, sigcache))
 			++num_srcs;
 
@@ -850,11 +872,14 @@ int git_diff_find_similar(
 
 		if ((tgt->flags & GIT_DIFF_FLAG__TO_SPLIT) != 0)
 			num_rewrites++;
+
+		if ((tgt->flags & GIT_DIFF_FLAG__TO_DELETE) != 0)
+			num_to_delete++;
 	}
 
-	/* if there are no candidate srcs or tgts, we're done */
+	/* If there are no candidate srcs or tgts, no need to find matches */
 	if (!num_srcs || !num_tgts)
-		goto cleanup;
+		goto split_and_delete;
 
 	src2tgt = git__calloc(num_deltas, sizeof(diff_find_match));
 	GIT_ERROR_CHECK_ALLOC(src2tgt);
@@ -995,6 +1020,7 @@ find_best_matches:
 				memset(&src->new_file, 0, sizeof(src->new_file));
 				src->new_file.path = src->old_file.path;
 				src->new_file.flags |= GIT_DIFF_FLAG_VALID_ID;
+				git_oid_clear(&src->new_file.id, diff->opts.oid_type);
 
 				num_updates++;
 
@@ -1020,6 +1046,7 @@ find_best_matches:
 				memset(&src->old_file, 0, sizeof(src->old_file));
 				src->old_file.path = src->new_file.path;
 				src->old_file.flags |= GIT_DIFF_FLAG_VALID_ID;
+				git_oid_clear(&src->old_file.id, diff->opts.oid_type);
 
 				src->flags &= ~GIT_DIFF_FLAG__TO_SPLIT;
 				num_rewrites--;
@@ -1089,15 +1116,20 @@ find_best_matches:
 		}
 	}
 
+split_and_delete:
 	/*
 	 * Actually split and delete entries as needed
 	 */
 
-	if (num_rewrites > 0 || num_updates > 0)
+	if (num_rewrites > 0 || num_updates > 0 || num_to_delete > 0) {
+		size_t apply_len = diff->deltas.length -
+		                   num_rewrites - num_to_delete;
+
 		error = apply_splits_and_deletes(
-			diff, diff->deltas.length - num_rewrites,
+			diff, apply_len,
 			FLAG_SET(&opts, GIT_DIFF_BREAK_REWRITES) &&
 			!FLAG_SET(&opts, GIT_DIFF_BREAK_REWRITES_FOR_RENAMES_ONLY));
+	}
 
 cleanup:
 	git__free(tgt2src);
